package de.hdm_stuttgart.mi.sd1.task1;

/**
 * <p>Handling first name and surname.</p>
 */
public class C_Switch {
    /**
     * <p>Switch  first name and surname.</p>
     *
     * <p>This method expects person names in reverse government agency style:
     * <span style="color:red;">First name</span> and <code style="color:blue;">surname</code> are being switched and
     * separated by a comma. These will be transformed back to their »regular« form. Examples:</p>
     *
     * <table class="goikTableDefaults">
     *     <caption>Switching Back to Regular</caption>
     *     <tbody>
     *         <tr>
     *             <th>Agency style</th>
     *             <th>Regular style</th>
     *         </tr>
     *         <tr>
     *             <td><code style="color:blue;">Davis</code>,<code style="color:red;">Miles</code></td>
     *             <td><code style="color:red;">Miles</code> <code style="color:blue;">Davis</code></td>
     *         </tr>
     *         <tr>
     *             <td><code style="color:blue;">Tolkien</code>,<code style="color:red;">John Ronald Reuel</code></td>
     *             <td><code style="color:red;">John Ronald Reuel</code> <code style="color:blue;">Tolkien</code></td>
     *         </tr>
     *     </tbody>
     * </table>
     *
     * <p>Whitespace shall be ignored. In the following examples the "<code style="color:red;">␣</code>" character
     * denotes whitespace:</p>
     *
     * <table class="goikTableDefaults">
     *     <caption>Whitespace handling</caption>
     *     <tbody>
     *         <tr>
     *             <th>Agency style</th>
     *             <th>Regular style</th>
     *         </tr>
     *         <tr>
     *             <td><code>Noether,<span style="color:red;">␣</span>Emmy</code></td>
     *             <td rowspan="3">Emmy Noether</td>
     *         </tr>
     *         <tr>
     *             <td><code>Noether,<span style="color:red;">␣␣␣</span>Emmy</code></td>
     *         </tr>
     *         <tr>
     *             <td><code><span style="color:red;">␣␣</span>Noether,<span style="color:red;">␣</span
     *               >Emmy<span style="color:red;">␣␣␣</span></code></td>
     *         </tr>
     *     </tbody>
     * </table>
     *
     * <p>Erroneous or completely absent input yields a <code>null</code> return value:</p>
     *
     * <table class="goikTableDefaults">
     *     <caption>Erroneous cases</caption>
     *     <tbody>
     *         <tr>
     *             <th>Agency Style</th>
     *             <th>Remark</th>
     *             <th>Result</th>
     *         </tr>
     *         <tr>
     *             <td><code>Rutherford John</code></td>
     *             <td><code>Missing comma</code></td>
     *             <td rowspan="7"><code>null</code></td>
     *         </tr>
     *         <tr>
     *             <td><code>Ronald Reuel,Tolkien,John</code></td>
     *             <td rowspan="2">Two commas</td>
     *         </tr>
     *         <tr>
     *             <td><code>Ronald Reuel<span style="color:red;">␣␣␣</span>,Tolkien,<span style="color:red;">␣</span>John</code></td>
     *         </tr>
     *         <tr>
     *             <td>,<code style="color:red;">John</code></td>
     *             <td rowspan ="2">Comma at start, no <code style="color:blue;">surname</code></td>
     *         </tr>
     *         <tr>
     *             <td>␣␣␣,␣<code style="color:red;">John</code></td>
     *         </tr>
     *         <tr>
     *             <td><code style="color:blue;">Updike</code>,</td>
     *             <td>Comma at end, no <code style="color:red;">first name</code></td>
     *         </tr>
     *         <tr>
     *             <td><code>null</code></td>
     *             <td>No input - no output</td>
     *         </tr>
     *     </tbody>
     * </table>
     *
     * <section class="implementationHints">
     *     <h4>Useful methods:</h4>
     *
     *     <ul>
     *         <li>{@link String#charAt(int)} retrieving character at given index.</li>
     *
     *         <li>{@link String#toCharArray()} converting a {@link String} into <code>char[]</code>.</li>
     *
     *         <li>{@link String#trim()} stripping off whitespace at beginning and end.</li>
     *
     *         <li>{@link String#split(String)} using <code>","</code> for regexp parameter.</li>
     *     </ul>
     * </section>
     *
     * @param agencyStyleName A person's name written as <code style="color:red">surname, first name</code>.
     * @return A regular person's name <code style="color:red">first name surname</code> or <code>null</code> in case of
     * error or complete lack of input.
     */
    static public String switchFirstAndSurname(final String agencyStyleName) {
        if (null == agencyStyleName || agencyStyleName.trim().equals("")) {
            return null;
        } else {
            final char[] stringCharacter = agencyStyleName.toCharArray();
            int commaCount = 0; // How many ',' chars are present?
            for (final char c: stringCharacter) {
                if (',' == c) {
                    commaCount++;
                }
            }
            if (1 == commaCount & // Exactly one comma
                    ',' != stringCharacter[0] & // Comma not at beginning
                    ',' != stringCharacter[agencyStyleName.length() -1]// Comma not at end
            ) {
                final String[] nameComponents = agencyStyleName.split(",");
                if (nameComponents[0].trim().equals("") ||    // Surname just whitespace
                        nameComponents[1].trim().equals(""))  // First name just whitespace
                {
                    return null;
                } else {
                    return nameComponents[1].trim().concat(" ").concat(nameComponents[0].trim());
                }
            } else { // No or multiple commas or at beginning or end
                return null;
            }
        }
    }

    private C_Switch() {/* Suppressing constructor Javadoc HTML generation */}
}
